# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from oxdb_utils import oxdb_makedir
from os.path import *
import cache
import extract


class BackupMovieFile:
    def __init__(self, parent):
        self.parent = parent

    def extractFrame(self, position, img_folder):
        return self.parent.extractFrame(position)

class Backup:
    def __init__(self, md5sum):
        self.md5sum = md5sum
        self.timelineFile = join(cache.cache_root, 'timeline', self.md5sum[:4], "%s.png" % self.md5sum)
        self.posterStill = join(cache.cache_root, 'posterStill', self.md5sum[:4], "%s.png" % self.md5sum)
        self.mini_movie_file = join(cache.cache_root, 'mini', self.md5sum[:4], "%s.avi" % self.md5sum)
        self.movieFile = BackupMovieFile(self)
        self.frameFolder = join(cache.cache_root, 'frame', self.md5sum[:4], self.md5sum)

    def exists(self):
        return exists(self.mini_movie_file)

    def timeline(self):
        return cache.loadTimeline(self)

    def frame(self, pos):
        return cache.loadFrame(self, pos)

    def posterStill(self, pos):
        return cache.loadPosterStill(self, pos)

    def flvClip(self, position):
        return cache.loadFlvClip(self, position)

    def extractFlvClip(self, inpoint, outpoint=-1):
        flash_folder = self.frameFolder
        movie_file = self.mini_movie_file
        position = inpoint.replace(':', '.')
        flash_movie = join(self.frameFolder, '%s.%s' % (position, 'flv'))
        width = 128
        height = int(width / self.frameAspect)
        height = height - height % 2
        inpoint = inpoint.replace('.', ':')
        if outpoint == -1:
            outpoint = shift_time(5000, inpoint)
        else:
            outpoint = outpoint.replace('.', ':')
        extract.extract_flash(movie_file, flash_movie, inpoint, outpoint, width, height, offset = 0)

    def extractFrame(self, position, img_folder=-1):
        img_folder = self.frameFolder
        movie_file = self.mini_movie_file
        oxdb_makedir(img_folder)
        position = position.replace('.', ':')
        extract.extract_frame(movie_file, position, img_folder, offset = 0, redo = False)

    def extractPosterStill(self, position):
        oxdb_makedir(dirname(self.posterStillFile))
        mFile = self.absolutePath
        if os.path.splitext(mFile)[-1] in ('.mov', '.mpg', '.mpeg'):
            mFile = self.mini_movie_file
        extract.extract_poster_still(mFile, self.posterStillFile, position)

