#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vi:si:et:sw=2:sts=2:ts=2
# OXDb Import client, crawls the filesystem and gathers information about
# movies
#
import md5
import os
import sys
import re
import urllib
import errno

from oxlib.normalize import normalizeName


def oxdb_filenameUmlaute(string):
  string = u"%s" % string
  string = string.replace(u'ae', u'ä')
  string = string.replace(u'oe', u'ö')
  string = string.replace(u'ue', u'ü')
  string = string.replace(u'Ae', u'Ä')
  string = string.replace(u'Oe', u'Ö')
  string = string.replace(u'Ue', u'Ü')
  return string

def oxdb_director(director):
    director = os.path.basename(os.path.dirname(director))
    if director.endswith('_'):
        director = "%s." % director[-1]
    director = ", ".join([normalizeName(d) for d in director.split('; ')])
    director = director.replace('Unknown Director', '')
    director = director.replace('Series', '')
    director = director.replace('Various Directors', '')
    return director

def oxdb_title(_title, searchTitle = False):
    '''
      normalize filename to get movie title
    '''
    _title = os.path.basename(_title).replace('. ', '_dot__space_')
    _title = _title.replace(' .', '_space__dot_')
    title = _title.split('.')[0]
    se = re.compile('Season (\d+).Episode (\d+)').findall(_title)
    if se:
        se = "S%02dE%02d" % (int(se[0][0]), int(se[0][1]))
        if 'Part' in _title.split('.')[-2] and 'Episode' not in _title.split('.')[-3]:
            stitle = _title.split('.')[-3]
        else:
            stitle = _title.split('.')[-2]
        if stitle.startswith('Episode '):
            stitle = '' 
        if searchTitle:
            title = '"%s" %s' % (title, stitle)
        else:
            title = '%s (%s) %s' % (title, se, stitle)
            title = title.strip()
    title = title.replace('_dot__space_', '. ')
    title = title.replace('_space__dot_', ' .')
    return title

def oxdb_year(data):
    return oxlib.findRe(data, '\.(\d{4})\.')

def oxdb_series_title(path):
    seriesTitle = u''
    if path.startswith('Series'):
        seriesTitle = os.path.basename(os.path.dirname(path))
    else:
        t = oxdb_title(path)
        if " (S" in t:
            seriesTitle = t.split(" (S")[0]
    return seriesTitle

def oxdb_episode_title(path):
    episodeTitle = u''
    ep = re.compile('.Episode \d+?\.(.*?)\.[a-zA-Z]').findall(path)
    if ep:
        episodeTitle = ep[0][0]
    return episodeTitle

def oxdb_season_episode(path):
    season = 0
    episode = 0
    path = os.path.basename(path)
    se = re.compile('Season (\d+).Episode (\d+)').findall(path)
    if se:
        season = int(se[0][0])
        episode = int(se[0][1])
    else:
        ep = re.compile('.Episode (\d+?)').findall(path)
        if ep:
            episode = int(ep[0][0])
    if season == 0 and episode == 0:
        se = re.compile('S(\d\d)E(\d\d)').findall(path)
        if se:
            season = int(se[0][0])
            episode = int(se[0][1])
    return (season, episode)

def oxdb_id(title, director):
  key = u"%s/%s" % (director,title)
  oxdb_id = '0x%s' % md5.new(key.encode('utf-8')).hexdigest()
  return oxdb_id

def imdb2oxdb(imdb):
  if imdb.startswith('0x') or imdb.startswith('ox') :
    return imdb
  return "0x%06X" % int(imdb)

def oxdb2imdb(oxdb):
  if len(oxdb) == 8:
    return "%07d" % float(oxdb)
  return oxdb

def trimString(string, length):
  if len(string) > length:
    string = string[:length - 13] + '...' + string[-10:]
  return string

def oxdb_makedir(folder):
  if not os.path.exists(folder):
    try:
      os.makedirs(folder)
    except os.error, e:
      if e.errno != errno.EEXIST or not os.path.isdir(folder):
        raise

