#!/usr/bin/python
# -*- coding: utf-8 -*-
# -*- Mode: Python; -*-
# vi:si:et:sw=2:sts=2:ts=2
#
import os
import time

import gobject
gobject.threads_init()

import pygst
pygst.require("0.10")
import gst
import Image

import singledecodebin


class FirstFrame:
  getShot = False
  length = 0
  height = 0
  def __init__(self, videofile, png='', nseconds=-1, width=128):
    s = ''' filesrc name=input 
          ! decodebin name=dbin 
          ! queue name =q 
          ! ffmpegcolorspace ! video/x-raw-rgb 
          ! fakesink name=output signal-handoffs=true
        '''

    self.width = width
    self.pipeline = gst.parse_launch(s)
    self.input  = self.pipeline.get_by_name('input')
    self.fakesink = self.pipeline.get_by_name('output')
    self.dbin = self.pipeline.get_by_name('dbin')
    self.bus = self.pipeline.get_bus()
    self.input.props.location = videofile

    self.pipeline.set_state(gst.STATE_PAUSED)
    self.pipeline.get_state()

    #length
    queue = self.pipeline.get_by_name('q')
    pads = queue.sink_pads()
    q = gst.query_new_duration(gst.FORMAT_TIME)
    for pad in pads:
      if pad.get_peer() and pad.get_peer().query(q):
        format, self.length = q.parse_duration()

    if nseconds>-1 and png:
      self.png(png, nseconds)

  def close(self):
    self.pipeline.set_state(gst.STATE_NULL)
    self.pipeline.get_state()

  def seek(self, nseconds):
    if(self.length and self.length < nseconds):
      nseconds = 0
    event = gst.event_new_seek(1.0, gst.FORMAT_TIME,
            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
            gst.SEEK_TYPE_SET, nseconds,
            gst.SEEK_TYPE_NONE, 0)

    res = self.fakesink.send_event(event)
    if res:
      self.pipeline.set_new_stream_time(0L)
    else:
        gst.error("seek to %r failed" % nseconds)

  def png(self, png, nseconds):
    self.png_frame = png
    self.pipeline.set_state(gst.STATE_PAUSED)
    self.pipeline.get_state()
    self.seek(nseconds)
    self.pipeline.set_state(gst.STATE_PLAYING)
    self.pipeline.get_state()

    ho = self.fakesink.connect("handoff", self.snapshot_png_handoff_cb)
    self.getShot = True
    while self.getShot:
      msg = self.bus.poll(gst.MESSAGE_ANY, gst.SECOND)
      if not msg:
        break
    self.fakesink.disconnect(ho)

  def snapshot_png_handoff_cb(self, sink, buffer, pad):
    if self.getShot:
      caps = sink.sink_pads().next().get_negotiated_caps()
      for s in caps:
        input_d = (s['width'], s['height'])
        output_d = self.scaleto(s['width'], s['height'])
      img = Image.fromstring('RGB',input_d,buffer)
      img = img.resize(output_d, Image.ANTIALIAS)
      img.save(self.png_frame)
      self.getShot=False

  def scaleto(self, width, height):
    height = int(self.width / (float(width) / height))
    height = height - height % 2
    self.height = height
    return (self.width, height)

def file_gnl_src(name, uri, caps, start, duration, offset, priority):
  src = singledecodebin.SingleDecodeBin(caps, uri)
  gnlsrc = gst.element_factory_make('gnlsource', name)
  gnlsrc.props.start = start
  gnlsrc.props.duration = duration
  gnlsrc.props.media_start = offset
  gnlsrc.props.media_duration = duration
  gnlsrc.props.priority = priority
  gnlsrc.add(src)
  
  return gnlsrc

class VideoEncoder:
  def __init__(self, output= 'ogg', width = 0, height = 0):
    self.bin = gst.Bin('VideoEncoder')
    self._queue_in = gst.element_factory_make("queue")
    self._queue_in.set_property("max-size-buffers",500)
    self._queue_in.set_property("max-size-time",0)
    self._queue_in.set_property("max-size-bytes",0)
    
    self._queue_out = gst.element_factory_make("queue")
    self._queue_out.set_property("max-size-buffers",500)
    
    self._identity = gst.element_factory_make("identity")
    self._identity.set_property('single-segment', True)
    
    self._ffmpegcolorspace = gst.element_factory_make("ffmpegcolorspace")
    self._videorate = gst.element_factory_make("videorate")
    self._videoscale = gst.element_factory_make("videoscale")
    self._videoscale.props.method = 1
    
    if output == 'flv':
      self.encoder = gst.element_factory_make("ffenc_flv")
      self.encoder.set_property('bitrate', 200000)
    elif output == 'mp4':
      self.encoder = gst.element_factory_make("x264enc")
      self.encoder.set_property('bitrate', 200)
    else:
      self.encoder = gst.element_factory_make("theoraenc")
      self.encoder.set_property("bitrate", 2000)
      self.encoder.set_property('sharpness', 1)
    
    self.bin.add(
                 self._queue_in,
                 self._identity,
                 self._ffmpegcolorspace,
                 self._videorate,
                 self._videoscale,
                 self.encoder,
                 self._queue_out
                )


    self._queue_in.link(self._videorate)

    fps = "25/1"
    caps = "video/x-raw-yuv, framerate=(fraction)%s" % fps
    gst_caps = gst.caps_from_string(caps)
    self._videorate.link(self._identity, gst_caps)

    self._identity.link(self._ffmpegcolorspace)
    
    self._ffmpegcolorspace.link(self._videoscale)
    
    if width:
      caps = "video/x-raw-yuv, width=%d, " % width
      if height:
        caps += "height=%d," % height
      caps += "pixel-aspect-ratio=(fraction)1/1"
      gst_caps = gst.caps_from_string(caps)
      self._videoscale.link(self.encoder, gst_caps)
    else:
      self._videoscale.link(self.encoder)
    self.encoder.link(self._queue_out)
    
    # Create GhostPads
    self.bin.add_pad(gst.GhostPad('sink', self._queue_in.get_pad('sink')))
    self.bin.add_pad(gst.GhostPad('src', self._queue_out.get_pad('src')))

class AudioEncoder:
  def __init__(self, output = 'ogg', samplerate = 44100, channels = 2):
    
    self.bin = gst.Bin('AudioEncoder')
    self._queue_in = gst.element_factory_make("queue")
    self._queue_in.set_property("max-size-buffers",500)
    self._queue_in.set_property("max-size-time",0)
    self._queue_in.set_property("max-size-bytes",0)
    
    self._queue_out = gst.element_factory_make("queue")
    self._queue_out.set_property("max-size-buffers",500)

    self._audiorate = gst.element_factory_make("audiorate")

    self._identity = gst.element_factory_make("identity")
    self._identity.set_property('single-segment', True)

    self._audioconvert = gst.element_factory_make("audioconvert")
    
    self._audioresample = gst.element_factory_make("audioresample")
    
    if output == 'flv':
      #MP3
      self.encoder = gst.element_factory_make("lame")
      self.encoder.set_property("bitrate", 32)
      self._mp3parse = gst.element_factory_make("mp3parse")
      self.bin.add(self._mp3parse)
      caps = "audio/x-raw-int,rate=%d,channels=%d" % (samplerate, channels)
    elif output == 'mp4':
      #AAC
      self.encoder = gst.element_factory_make("faac")
      #self.encoder.set_property("bitrate", 32)
      caps = "audio/x-raw-int,rate=%d,channels=%d" % (samplerate, channels)
    else:
      #Vorbis
      self.encoder = gst.element_factory_make("vorbisenc")
      self.encoder.set_property("quality", 0)
      caps = "audio/x-raw-float,rate=%d,channels=%d" % (samplerate, channels)
    
    self.bin.add(
                 self._queue_in,
                 self._audiorate,
                 self._identity,
                 self._audioconvert,
                 self._audioresample,
                 self.encoder,
                 self._queue_out
                )
    
    self._queue_in.link(self._audiorate)
    self._audiorate.link(self._identity)
    self._identity.link(self._audioconvert)

    self._audioconvert.link(self._audioresample)
    
    arate = gst.caps_from_string(caps)
    self._audioresample.link(self.encoder, arate)
    
    if output == 'flv':
      self.encoder.link(self._mp3parse)
      self._mp3parse.link(self._queue_out)
    else:
      self.encoder.link(self._queue_out)
    
    # Create GhostPads
    self.bin.add_pad(gst.GhostPad('sink', self._queue_in.get_pad('sink')))
    self.bin.add_pad(gst.GhostPad('src', self._queue_out.get_pad('src')))

class ExtractClip:
  acaps = gst.caps_from_string("audio/x-raw-int;audio/x-raw-float")
  vcaps = gst.caps_from_string("video/x-raw-rgb; video/x-raw-yuv")  
  width = 0
  samplerate = 44100
  channels = 2
  
  def __init__(self, location, outfile, offset, duration, width, height, samplerate, channels):
    self.mainloop = gobject.MainLoop()
    
    if outfile.endswith('flv'):
      self.outputformat = 'flv'
    elif outfile.endswith('mp4'):
      self.outputformat = 'mp4'
    else:
      self.outputformat = 'ogg'
    
    self.width = width
    self.height = height
    self.samplerate = samplerate
    self.channels = channels
    self._pipeline = gst.Pipeline()
    
    self.bus = self._pipeline.get_bus()
    self.bus.add_signal_watch()
    self.watch_id = self.bus.connect("message", self.onBusMessage)
    
    self.atimeline = gst.element_factory_make("gnlcomposition", "ATimeline")
    self.atimeline.connect("pad-added", self._on_new_apad)
    self._pipeline.add(self.atimeline)

    self.vtimeline = gst.element_factory_make("gnlcomposition", "VTimeline")
    self.vtimeline.connect("pad-added", self._on_new_vpad)
    self._pipeline.add(self.vtimeline)
    
    #Mux streams
    if self.outputformat == 'flv':
      self._mux =  gst.element_factory_make("ffmux_flv")
    elif self.outputformat == 'mp4':
      self._mux =  gst.element_factory_make("ffmux_mp4")
    else:
      self._mux =  gst.element_factory_make("oggmux")
    self._pipeline.add(self._mux)
    
    location = "file://" + os.path.abspath(location)
    
    self._filesink = gst.element_factory_make("filesink")
    self._filesink.set_property("location", outfile)
    self._pipeline.add(self._filesink)
    

    self._mux.link(self._filesink)
        
    media_start = offset
    media_duration = duration
    start = 0 * gst.SECOND
    
    name = os.path.basename(location)

    #video source
    '''
    vsrc = gst.element_factory_make("gnlfilesource", "%s_v" % name)
    vsrc.props.location = location
    vsrc.props.caps = self.vcaps
    vsrc.props.media_start = media_start
    vsrc.props.media_duration = media_duration + gst.SECOND / 50
    vsrc.props.start = start
    vsrc.props.duration = media_duration
    '''
    vsrc = file_gnl_src("%s_v" % name, 
                        location, self.vcaps, start, 
                        media_duration + gst.SECOND / 25, media_start, 0)
    self.vtimeline.add(vsrc)

    #audio source
    '''
    asrc = gst.element_factory_make("gnlfilesource", "%s_a" % name)
    asrc.props.location = location
    asrc.props.caps = self.acaps
    asrc.props.media_start = media_start
    asrc.props.media_duration = media_duration
    asrc.props.start = start
    asrc.props.duration = media_duration
    '''
    asrc = file_gnl_src("%s_a" % name, 
                        location, self.acaps, start, 
                        media_duration, media_start, 0)

    self.atimeline.add(asrc)
    
    self.endpos = media_start + media_duration
  
    self.vencoder = VideoEncoder(self.outputformat, self.width, self.height)
    self._pipeline.add(self.vencoder.bin)
    self.vencoder.bin.link(self._mux)
    
    self.aencoder = AudioEncoder(self.outputformat, self.samplerate, self.channels)
    self._pipeline.add(self.aencoder.bin)

    self.aencoder.bin.link(self._mux)
    
    
  def extract(self):
    self._pipeline.set_state(gst.STATE_PAUSED)
    self._pipeline.get_state()
    self._pipeline.set_state(gst.STATE_PLAYING)
    self._pipeline.get_state()
    self.mainloop.run()

  def onBusMessage(self, bus, message):
    if message.src == self._pipeline and message.type == gst.MESSAGE_EOS:
      self.quit()
    return True
  
  def quit(self):
    self._pipeline.set_state(gst.STATE_NULL)
    self._pipeline.get_state()
    self.mainloop.quit()
  
  def _on_new_vpad(self, element, pad):
    pad.link(self.vencoder.bin.get_pad("sink"))
  
  def _on_new_apad(self, element, pad):
    pad.link(self.aencoder.bin.get_pad("sink"))
    
if __name__ == "__main__":
  import sys
  width = 128
  height = 80
  samplerate = 44100
  channels = 1
  inputFile = sys.argv[1]
  outputFile = sys.argv[2]
  offset = int(float(sys.argv[3]) * gst.MSECOND)
  duration = int(float(sys.argv[4]) * gst.MSECOND)
  
  frameName = "%s.jpg" % os.path.splitext(outputFile)[0]
  f = FirstFrame(inputFile, frameName, offset, width)
  height = f.height
  clip = ExtractClip(inputFile, outputFile, offset, duration, width, height, samplerate, channels)
  clip.extract()
