#!/usr/bin/python
# -*- coding: utf-8 -*-
# -*- Mode: Python; -*-
# vi:si:et:sw=2:sts=2:ts=2
#
import os
import time
import shutil

import gobject
gobject.threads_init()

import pygst
pygst.require("0.10")
import gst
import Image

class PosterStill:
  getShot = False
  length = 0
  height = 0
  def __init__(self, videofile, png='', nseconds=-1, height=128):
    s = ''' filesrc name=input 
          ! decodebin name=dbin 
          ! queue name =q 
          ! ffmpegcolorspace ! video/x-raw-rgb 
          ! fakesink name=output signal-handoffs=true
        '''

    self.height = height
    self.pipeline = gst.parse_launch(s)
    self.input  = self.pipeline.get_by_name('input')
    self.fakesink = self.pipeline.get_by_name('output')
    self.dbin = self.pipeline.get_by_name('dbin')
    self.bus = self.pipeline.get_bus()
    self.input.props.location = videofile

    self.pipeline.set_state(gst.STATE_PAUSED)
    self.pipeline.get_state()

    #length
    queue = self.pipeline.get_by_name('q')
    pads = queue.sink_pads()
    q = gst.query_new_duration(gst.FORMAT_TIME)
    for pad in pads:
      if pad.get_peer() and pad.get_peer().query(q):
        format, self.length = q.parse_duration()

    if nseconds>-1 and png:
      self.png(png, nseconds)

  def close(self):
    self.pipeline.set_state(gst.STATE_NULL)
    self.pipeline.get_state()

  def seek(self, nseconds):
    if(self.length and self.length < nseconds):
      nseconds = 0
    event = gst.event_new_seek(1.0, gst.FORMAT_TIME,
            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
            gst.SEEK_TYPE_SET, nseconds,
            gst.SEEK_TYPE_NONE, 0)

    res = self.fakesink.send_event(event)
    if res:
      self.pipeline.set_new_stream_time(0L)
    else:
        gst.error("seek to %r failed" % nseconds)

  def png(self, png, nseconds):
    self.png_frame = png
    self.pipeline.set_state(gst.STATE_PAUSED)
    self.pipeline.get_state()
    self.seek(nseconds)
    self.pipeline.set_state(gst.STATE_PLAYING)
    self.pipeline.get_state()

    ho = self.fakesink.connect("handoff", self.snapshot_png_handoff_cb)
    self.getShot = True
    while self.getShot:
      msg = self.bus.poll(gst.MESSAGE_ANY, gst.SECOND)
      if not msg:
        break
    self.fakesink.disconnect(ho)

  def snapshot_png_handoff_cb(self, sink, buffer, pad):
    if self.getShot:
      caps = sink.sink_pads().next().get_negotiated_caps()
      for s in caps:
        input_d = (s['width'], s['height'])
        output_d = self.scaleto(s['width'], s['height'])
      img = Image.fromstring('RGB',input_d,buffer)
      img = img.resize(output_d, Image.ANTIALIAS)
      img.save(self.png_frame)
      self.getShot=False

  def scaleto(self, width, height):
    width = int(self.height * (float(width) / height))
    width = width - width%2
    self.width = width
    return (self.width, self.height)

class PosterStillMplayer:
  length = 0
  height = 0
  def __init__(self, videofile, png='', nseconds=-1, height=128):
    import warnings
    warnings.filterwarnings("ignore", "tempnam")
    
    self.height = height
    self.videofile = videofile
    self.png_frame = png
    
    framedir = os.tempnam()
    os.mkdir(framedir)
    os.chdir(framedir)
    mplayer_options = ''
    mplayer_options += " '%s'" % self.videofile
    mplayer_options += " -ss '%s' -frames 2" % self.msec2time(nseconds)
    mplayer_options += " -vo png "
    mplayer_options += " -ao null"
    mplayer = "mplayer %s >/dev/null 2>&1" % mplayer_options
    
    #execute mplayer
    os.system (mplayer.encode('utf-8'))
    #get a last decoded frame
    files = os.listdir(framedir)
    if files:
      #scale and save it
      self.scaleImage(files[-1])
    shutil.rmtree(framedir)
  
  def msec2time(self, msec):
    import time
    msec_string = "%s" % msec
    ms = ",%s" % msec_string[-3:]
    sec = float(msec) / 1000
    return time.strftime("%H:%M:%S", time.gmtime(sec)) + ms
  
  def scaleImage(self, tmpImage):
    img = Image.open(tmpImage)
    output_d = self.scaleto(img.size[0], img.size[1])
    img = img.resize(output_d, Image.ANTIALIAS)
    img.save(self.png_frame)
  
  def scaleto(self, width, height):
    width = int(self.height * (float(width) / height))
    width = width - width%2
    self.width = width
    return (self.width, self.height)

if __name__ == "__main__":
  import sys
  height = 128
  inputFile = sys.argv[1]
  #mov is broken somehow
  if inputFile.endswith('.mov'):
    sys.exit()
  outputFile = sys.argv[2]
  offset = int(float(sys.argv[3]) * gst.MSECOND)
  f = PosterStill(inputFile, outputFile, offset, height)
  #f = PosterStillMplayer(inputFile, outputFile, offset, height)
